%
% run glint, based on a settings file
%
% The glint settings file is formatted as follows:
% 1:glimmer command line with arguments separated by commas NOT spaces
% 2:a comma-separated triple of the C cycle (initializer,uppperbound,step), step must be multiple of 4
% 3:number of samples to generate for each gprocess calculation
% 4:epsilon termination criterion
%
% inputs are : 
%
% 	settings_filename = file described above, only uses the first line of settings file
%	numsamples = number of times it runs the glimmer algorithm to generate a sparse stress sample
%	func_eps = not used right now
%	gp_at_step = number of outer loop iters before
%	stepper =  the values of C to go through
%	learning_rate = like a threshold
%
% outputs are :
%
% [S D C L X] 
% where S = set of final sparse stresses
%       D = set of smoothed differences
%       M = the final means of the process
%       C = set of iteration counts
%       L = cost of a single iteration
%	X = layouts after termination criteria
%	step_size_out = the size of the step size
%	n = outer loops
%	fs = final stress 
%	rp = realphi 
%
function [S D M Cs L X step_size_out n fs rp] = glint( settings_filename, N, numsamples, gp_at_step, K0, phi )

	if nargin < 4
		
		gp_at_step = 3;
	end

	% setup gaussian process variables
	s2coeff = 2;
	meanfunc = [];
	hyp1.mean = [];
	covfunc = @covSEiso;
	hyp1.cov = [2 2];
	likfunc = @likGauss;
	hyp1.lik = log(0.1);

	% open the settings file
	
	settings_fid = fopen( settings_filename, 'r' );
	
	% build the first line
	
	cmdline_raw = fgetl(settings_fid);
	stressline_raw = fgetl(settings_fid);
	cmdline_cell = strsplit(cmdline_raw,',');
	stressline_cell = strsplit(stressline_raw,',');
	cmdline_true = '';
	stressline_true = '';
	for i = 1:length(cmdline_cell)
		
		cmdline_true = cstrcat(cmdline_true, '''', cmdline_cell{i}, ''' ');
	end
	for i = 1:length(stressline_cell)
		
		stressline_true = cstrcat(stressline_true, '''', stressline_cell{i}, ''' ');
	end
	
	% % build the stepper
	% 
	% if nargin < 5
	% 	stepper_raw = fgetl( settings_fid );
	% 	stepper_arr = sscanf( stepper_raw, '%d,%d,%d');
	% 	stepper = stepper_arr(1):stepper_arr(3):stepper_arr(2);
	% end
	% 
	sparse_stress_samples = [];
	full_stress_samples = [];
	D = [];
	Cs = [];
	L = 0;
	
	% close the file
	
	fclose(settings_fid);
	
	steps_in = 0;
	real_steps_in = 0;
	
	step_size = K0;
	step_start = K0;
	C = step_start;
	
	has_optimized = 0;
	first_run = 1;
	step_size_ok = 0;
	
	stepcollection = [];
	
	while round(step_size) > 0 && C <= N
		
		% disp(C);
		% fflush(stdout);
		
		steps_in = steps_in + 1;
		real_steps_in = real_steps_in + 1;
		
		stepcollection(real_steps_in) = C;
		
		for samplenum = 1:numsamples
			
			cmdline_inner = cmdline_true;
			
			if first_run
				
				cmdline_inner = strcat( cmdline_inner, ' -m');
			end
			
			cmdline_inner = strcat( cmdline_inner, ' -c');
			
			if samplenum == numsamples
				cmdline_inner = strcat( cmdline_inner, ' -o glint_temp.csv');
			end
			
			if C > step_start
				
				cmdline_inner = strcat( cmdline_inner, ' -Q glint_temp.csv');
			end
			
			% fprintf(cstrcat( cmdline_inner, sprintf(' -C %d\n',C)));
			[STATUS,OUTPUT]=system( cstrcat( cmdline_inner, sprintf(' -C %d',C)));
			% fprintf(OUTPUT);
			% fprintf('C=%d\n',C)
			OUTPUT_split = strsplit(OUTPUT, "\n");
			% parse output arguments
			
			for outcheck = 1:length( OUTPUT_split)
				
				if length( OUTPUT_split{outcheck} ) < 1
					continue;
				end
				
				inner_split = strsplit( OUTPUT_split{outcheck}, '=');
				if strcmp( inner_split{1}, 'C' )
					Cs(real_steps_in, samplenum) = sscanf( inner_split{2}, '%d' );
				elseif strcmp(inner_split{1}, 'L')
					L = sscanf( inner_split{2}, '%f' );
				elseif strcmp(inner_split{1}, 'sstress')
					fflush(stdout);
					sparse_stress_samples(real_steps_in, samplenum) = sscanf( inner_split{2}, '%f' );
				elseif strcmp(inner_split{1}, 'fullstress')
					full_stress_samples(real_steps_in, samplenum) = sscanf( inner_split{2}, '%f' );
				end
			end
			
			if first_run
				
				first_run = 0;
				% step_size = ceil( (Cs(steps_in,end) * phi) / N );
				step_size = ceil(log10(N));
				rp = (step_size * N) / Cs(steps_in,end);
				step_size_out = step_size;
			end
		end
		
		% perform gaussian process regression on output
		
		if steps_in >= gp_at_step
		
			y = reshape( sparse_stress_samples(((real_steps_in-steps_in) + 1):real_steps_in,:)', steps_in * numsamples, 1);
			x = reshape( repmat( stepcollection(((real_steps_in-steps_in) + 1):real_steps_in), numsamples, 1), numsamples*steps_in, 1 );
			
			
			% y = reshape( sparse_stress_samples(1:steps_in,:)', steps_in * numsamples, 1);
			% x = reshape( repmat( (1:steps_in), numsamples, 1), numsamples*steps_in, 1 );
			
			% dd = filter( [1 -1], 1, y(numsamples:numsamples:end) );
			% ddd = filter( ones(1,gp_at_step)/gp_at_step,1,dd);
			% D(steps_in)=dd(end);
			
			if ~has_optimized
				
				has_optimized = 1;
				hyp1 = minimize( hyp1, @gp, -100, @infExact, meanfunc, covfunc, likfunc, x, y);
			end
			[m s2] = gp(hyp1, @infExact,  meanfunc, covfunc, likfunc, x, y, x(numsamples:numsamples:end));
			D( steps_in ) = m(end) - m(end - (gp_at_step-1));
			M = m;
			
			% see if we've made sufficient progress
			if ~step_size_ok 
				if abs(D(steps_in)) < s2coeff*sqrt(s2(end))
					% if D(steps_in) < 0
					step_size = min( 2*(ceil(log10(N))), step_size * 2);
					steps_in = 0;
					has_optimized = 0;
				else
					step_size_ok = 1;
				end
			else
				% if D(steps_in) < s2(end)
				% 	step_size = 0;
				% else
				% 	step_size = step_size * min(2,D(steps_in)/s2(end));
				% end
				if abs(D(steps_in)) < s2coeff*sqrt(s2(end))
					step_size = 0;
				end
			end
			% step_size = step_size * max(min( D(steps_in) * (1/sqrt(s2(end))), 1 ),learning_rate);
			
			if( steps_in > 0)
				fprintf('step_size = %f D = %f s2 = %f\n', step_size,abs(D(steps_in)),s2coeff*sqrt(s2(end)));
			else
				fprintf('step_size = %f\n', step_size);
			end
				
			fflush(stdout);
			
			% D( steps_in ) = s2(end);
		end
		
		C = C + round(step_size);
	end
	
	% compute the final stress

	[STATUS,OUTPUT]=system( cstrcat( stressline_true, ' -I -i glint_temp.csv'));
	OUTPUT_split = strsplit(OUTPUT, "\n");
	
	for outcheck = 1:length( OUTPUT_split)
		
		if length( OUTPUT_split{outcheck} ) < 1
			continue;
		end
		
		inner_split = strsplit( OUTPUT_split{outcheck}, '=');
		if strcmp(inner_split{1}, 'fullstress')
			fs = sscanf( inner_split{2}, '%f' );
		end
	end
	
	% parse and open the output file
	
	X = csvread('glint_temp.csv');
	system( 'rm glint_temp.csv' );
	S = sparse_stress_samples;
	n = real_steps_in;